/*
 * Copyright 2006-2009 Luc Verhaegen.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * Our own X extension to push mpeg-2 data over the X protocol.
 *
 * People are blind and stupid, and infatuated with DRM, and therefor
 * completely messed up XvMC.
 */
#include <X11/Xlib.h>
#include <X11/extensions/Xv.h>
#include <X11/extensions/Xvlib.h>
#include <X11/extensions/vldXvMC.h>

#include <stdio.h>

#include <X11/Xlibint.h>
#include <X11/Xproto.h>
#include <X11/extensions/Xext.h>
#include <X11/extensions/extutil.h>

#include "xvmce_proto.h"
#include "xvmce.h"

#define XvMCECheckExtension(dpy, info, val) \
    XextCheckExtension(dpy, info, XVMCE_EXT_NAME, val)

static XExtensionInfo *XvMCEExtInfo = NULL;

static int XvMCEExtCloseDisplay(Display *dpy, XExtCodes *codes);
XEXT_GENERATE_CLOSE_DISPLAY(XvMCEExtCloseDisplay, XvMCEExtInfo)

XExtDisplayInfo *XvMCEFindDisplay(Display *dpy);
static XExtensionHooks XvMCEExtHooks = {
    NULL, /* create_gc */
    NULL, /* copy_gc */
    NULL, /* flush_gc */
    NULL, /* free_gc */
    NULL, /* create_font */
    NULL, /* free_font */
    XvMCEExtCloseDisplay,
    NULL, /* wire_to_event */
    NULL, /* event_to_wire */
    NULL, /* error */
    NULL, /* error_string */
};
XEXT_GENERATE_FIND_DISPLAY(XvMCEFindDisplay, XvMCEExtInfo, XVMCE_EXT_NAME,
                           &XvMCEExtHooks, 0, NULL)

#define XvMCEGetReq(name, req) \
        WORD64ALIGN\
	if ((dpy->bufptr + SIZEOF(xvmce##name##Req)) > dpy->bufmax)\
		_XFlush(dpy);\
	req = (xvmce##name##Req *)(dpy->last_req = dpy->bufptr);\
	req->reqType = info->codes->major_opcode;\
        req->xvmceReqType = xvmce_##name; \
        req->length = (SIZEOF(xvmce##name##Req))>>2;\
	dpy->bufptr += SIZEOF(xvmce##name##Req);\
	dpy->request++

#define XvMCEGetReqExtra(name, req, len)             \
        WORD64ALIGN\
	if ((dpy->bufptr + SIZEOF(xvmce##name##Req)) > dpy->bufmax)\
		_XFlush(dpy);\
	req = (xvmce##name##Req *)(dpy->last_req = dpy->bufptr);\
	req->reqType = info->codes->major_opcode;\
        req->xvmceReqType = xvmce_##name; \
        req->length = (SIZEOF(xvmce##name##Req) + len + 3)>>2;\
	dpy->bufptr += SIZEOF(xvmce##name##Req);\
	dpy->request++

/*
 *
 */
Status
XvMCEQueryExtension(Display *dpy, int *Version, int *Release,
                   int *event_basep, int *error_basep)
{
    XExtDisplayInfo *info = XvMCEFindDisplay(dpy);
    xvmceQueryExtensionReq *req;
    xvmceQueryExtensionReply reply;
    Status status;

    XvMCECheckExtension(dpy, info, BadImplementation);

    LockDisplay(dpy);

    XvMCEGetReq(QueryExtension, req);

    if (_XReply(dpy, (xReply *)&reply, 0, xFalse)) {
        *Version = reply.Version;
        *Release = reply.Release;

        if ((*Version != XVMCE_VERSION) || (*Release < XVMCE_RELEASE)) {
            fprintf(stderr, "Incompatible %s version found.\n", XVMCE_EXT_NAME);

            status = BadImplementation;
        }

        *event_basep = info->codes->first_event;
	*error_basep = info->codes->first_error;

        status = Success;
    } else {
        fprintf(stderr, "%s; Xreply failed.\n", __func__);
        status = BadAccess;
    }

    UnlockDisplay(dpy);
    SyncHandle();

    return status;
}

/*
 *
 */
Status
XvMCEQMatrixSend(Display *dpy, XvPortID port, char type,
		 const unsigned char qmatrix[64])
{
    XExtDisplayInfo *info = XvMCEFindDisplay(dpy);
    xvmceQMatrixSendReq *req;

    XvMCECheckExtension(dpy, info, BadImplementation);

    LockDisplay(dpy);

    XvMCEGetReqExtra(QMatrixSend, req, 64);

    req->Port = port;
    req->Type = type;

    _XSend(dpy, (char *) qmatrix, 64);

    UnlockDisplay(dpy);
    SyncHandle();

    return Success;
}

/*
 *
 */
Status
XVMCEBufferSliceInit(Display *dpy, XvPortID port, XID Buffer,
		     CARD8 PictureType, CARD8 SliceFlags,
		     Bool HasBackwardsReference, XID BackwardsReference,
		     Bool HasForwardsReference, XID ForwardsReference,
		     CARD8 MVRange_BackwardsHorizontal,
		     CARD8 MVRange_BackwardsVertical,
		     CARD8 MVRange_ForwardsHorizontal,
		     CARD8 MVRange_ForwardsVertical)
{
    XExtDisplayInfo *info = XvMCEFindDisplay(dpy);
    xvmceBufferSliceInitReq *req;


    XvMCECheckExtension(dpy, info, BadImplementation);

    LockDisplay(dpy);

    XvMCEGetReq(BufferSliceInit, req);

    req->Port = port;
    req->Buffer = Buffer;
    req->PictureType = PictureType;
    req->SliceFlags = SliceFlags;

    req->ReferencesPresent = 0;
    if (HasBackwardsReference)
	req->ReferencesPresent |= XVMCE_BACKWARDS_PRESENT;
    if (HasForwardsReference)
	req->ReferencesPresent |= XVMCE_FORWARDS_PRESENT;

    req->BackwardsReferenceBuffer = BackwardsReference;
    req->ForwardsReferenceBuffer = ForwardsReference;

    req->MVRange_BackwardsHorizontal = MVRange_BackwardsHorizontal;
    req->MVRange_BackwardsVertical = MVRange_BackwardsVertical;
    req->MVRange_ForwardsHorizontal = MVRange_ForwardsHorizontal;
    req->MVRange_ForwardsVertical = MVRange_ForwardsVertical;

    UnlockDisplay(dpy);
    SyncHandle();

    return Success;
}

/*
 *
 */
Status
XVMCEBufferSliceSend(Display *dpy, XvPortID port, XID Buffer,
		     int SliceCount, int SliceSize, char *Slice)
{
    XExtDisplayInfo *info = XvMCEFindDisplay(dpy);
    xvmceBufferSliceSendReq *req;

    XvMCECheckExtension(dpy, info, BadImplementation);

    LockDisplay(dpy);

    XvMCEGetReqExtra(BufferSliceSend, req, SliceSize);

    req->Port = port;
    req->Buffer = Buffer;
    req->SliceCount = SliceCount;
    req->SliceLength = SliceSize;

    _XSend(dpy, Slice, SliceSize);

    UnlockDisplay(dpy);
    SyncHandle();

    return Success;
}
