/*
 * Copyright 2006-2009 Luc Verhaegen.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * Protocol definition for unichrome xvmce.
 */
#ifndef XVMCE_PROTO_H
#define XVMCE_PROTO_H

#define XVMCE_EXT_NAME "Unichrome XvMC-E"

#define XVMCE_VERSION 0
#define XVMCE_RELEASE 1

#define xvmce_QueryExtension    0
#define xvmce_QMatrixSend       1
#define xvmce_BufferSliceInit   2
#define xvmce_BufferSliceSend   3

/*
 * Version, are we compatible?
 */
typedef struct {
    CARD8   reqType;
    CARD8   xvmceReqType; /* xvmce_QueryExtension */
    CARD16  length B16;
} xvmceQueryExtensionReq;
#define sz_xvmceQueryExtensionReq 4

typedef struct {
    BYTE    type;   /* X_Reply */
    BYTE    pad0;
    CARD16  sequenceNumber B16;
    CARD32  length B32;
    CARD32  Version B32;
    CARD32  Release B32;
    CARD32  pad2 B32;
    CARD32  pad3 B32;
    CARD32  pad4 B32;
    CARD32  pad5 B32;
} xvmceQueryExtensionReply;
#define sz_xvmceQueryExtensionReply 32

/*
 * QMatrixSend, upload a Quantization matrix to the MPEG engine.
 */
typedef struct {
    CARD8     reqType;
    CARD8     xvmceReqType; /* xvmce_QMatrixSend */
    CARD16    length B16;
    CARD32    Port B32;
#define XVMCE_QMATRIX_INTRA           0
#define XVMCE_QMATRIX_NONINTRA        1
#define XVMCE_QMATRIX_INTRA_CHROMA    2
#define XVMCE_QMATRIX_NONINTRA_CHROMA 3
    CARD32    Type B32;
} xvmceQMatrixSendReq;
#define sz_xvmceQMatrixSendReq 12

/*
 *
 */
typedef struct {
    CARD8     reqType;
    CARD8     xvmceReqType; /* xvmce_BufferSliceInit */
    CARD16    length B16;
    CARD32    Port B32;
    CARD32    Buffer B32;

#define XVMCE_PICTURETYPE_FIELD_TOP      0x01
#define XVMCE_PICTURETYPE_FIELD_BOTTOM   0x02
#define XVMCE_PICTURETYPE_FRAME          0x03 /* frame, so progressive. */
#define XVMCE_PICTURETYPE_STRUCTURE_MASK 0x03

#define XVMCE_PICTURETYPE_FRAMETYPE_I    0x01 << 2
#define XVMCE_PICTURETYPE_FRAMETYPE_P    0x02 << 2
#define XVMCE_PICTURETYPE_FRAMETYPE_B    0x03 << 2
#define XVMCE_PICTURETYPE_FRAMETYPE_MASK 0x03 << 2

#define XVMCE_PICTURETYPE_SCAN_ZIGZAG    0x00 << 4
#define XVMCE_PICTURETYPE_SCAN_ALTERNATE 0x01 << 4
#define XVMCE_PICTURETYPE_SCAN_MASK      0x01 << 4
    CARD8     PictureType;

#define XVMCE_SLICEFLAGS_PREDICTION_FIELD        0x00
#define XVMCE_SLICEFLAGS_PREDICTION_FRAME        0x01
#define XVMCE_SLICEFLAGS_PREDICTION_MASK         0x01

#define XVMCE_SLICEFLAGS_FIELD_FIRST_BOTTOM      0x00 << 1
#define XVMCE_SLICEFLAGS_FIELD_FIRST_TOP         0x01 << 1
#define XVMCE_SLICEFLAGS_FIELD_FIRST_MASK        0x01 << 1

#define XVMCE_SLICEFLAGS_INTRA_HAS_MV            0x01 << 2

#define XVMCE_SLICEFLAGS_INTRA_DC_TABLE_MASK     0x01 << 3

#define XVMCE_SLICEFLAGS_INTRA_PRECISION_MASK    0x03 << 4

#define XVMCE_SLICEFLAGS_QUANTIZATION_TABLE_MASK 0x01 << 6
    CARD8     SliceFlags;

    CARD8     PadByte;
#define XVMCE_BACKWARDS_PRESENT 1
#define XVMCE_FORWARDS_PRESENT  2
    CARD8     ReferencesPresent;

    CARD32    BackwardsReferenceBuffer B32;
    CARD32    ForwardsReferenceBuffer B32;

    CARD8     MVRange_BackwardsHorizontal;
    CARD8     MVRange_BackwardsVertical;
    CARD8     MVRange_ForwardsHorizontal;
    CARD8     MVRange_ForwardsVertical;
} xvmceBufferSliceInitReq;
#define sz_xvmceBufferSliceInitReq 28

/*
 *
 */
typedef struct {
    CARD8     reqType;
    CARD8     xvmceReqType; /* xvmce_BufferSliceSend */
    CARD16    length B16;
    CARD32    Port B32;
    CARD32    Buffer B32;
    CARD32    SliceCount B32;
    CARD32    SliceLength B32;
} xvmceBufferSliceSendReq;
#define sz_xvmceBufferSliceSendReq 20

#endif /* XVMCE_PROTO_H */
