/*
 * Copyright 2004-2009 Luc Verhaegen.
 * Copyright 2004-2005 The Unichrome Project  [unichrome.sf.net]
 * Copyright 1998-2003 VIA Technologies, Inc. All Rights Reserved.
 * Copyright 2001-2003 S3 Graphics, Inc. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef _VIA_DRIVER_H_
#define _VIA_DRIVER_H_ 1

#include "xf86.h"

#include "xf86Pci.h"
#include "xf86PciInfo.h"
#include "xf86_OSproc.h"
#include "compiler.h"
#include "micmap.h"

#include "xf86cmap.h"

#include "via_memory.h"

#ifdef XvExtension
#include "via_video.h"
#endif

#define DRIVER_NAME     "via"
#define VERSION_MAJOR   0
#define VERSION_MINOR   2
#define PATCHLEVEL      0
#define VIA_VERSION     ((VERSION_MAJOR<<24) | (VERSION_MINOR<<16) | PATCHLEVEL)

#ifdef XSERVER_LIBPCIACCESS
typedef struct pci_device *pciVideoPtr;
#endif

#define VIA_CBUFFERSIZE         512

/* For pVia->ActiveDevice */
#define VIA_DEVICE_NONE  0x00
#define VIA_DEVICE_CRT   0x01
#define VIA_DEVICE_LCD   0x02
#define VIA_DEVICE_TV    0x04
#define VIA_DEVICE_DVI   0x08

/* For pVia->MemClk */
#define VIA_MEM_DDR_200   0x00
#define VIA_MEM_DDR_266   0x01
#define VIA_MEM_DDR_333   0x02
#define VIA_MEM_DDR_400   0x03
#define VIA_MEM_DDR2_400  0x04
#define VIA_MEM_DDR2_533  0x05
#define VIA_MEM_DDR2_667  0x06
#define VIA_MEM_DDR2_800  0x07
#define VIA_MEM_DDR2_1066 0x08
#define VIA_MEM_END      0x09
#define VIA_MEM_NONE     0xFF

/* For Scratch->TVStandard */
#define TVSTANDARD_NONE  0x00
#define TVSTANDARD_NTSC  0x01
#define TVSTANDARD_PAL   0x02

struct ViaScratch {
    int  VideoRam;
    CARD8  MemType;
    CARD8  ActiveDevice;
    CARD8  PanelSize;
    CARD8  TVStandard;
};

enum VIAAccelMethod {
    VIA_ACCEL_NONE,
    VIA_ACCEL_SHADOWFB,
    VIA_ACCEL_XAA,
    VIA_ACCEL_EXA /* not yet */
};

typedef struct {
    CARD8   SR08, SR0A, SR0F;

    /*   extended Sequencer registers */
    CARD8   SR10, SR11, SR12, SR13, SR14, SR15, SR16;
    CARD8   SR17, SR18, SR19, SR1A, SR1B, SR1C, SR1D, SR1E;
    CARD8   SR1F, SR20, SR21, SR22, SR23, SR24, SR25, SR26;
    CARD8   SR27, SR28, SR29, SR2A, SR2B, SR2C, SR2D, SR2E;
    CARD8   SR2F, SR30, SR31, SR32, SR33, SR34, SR40, SR41;
    CARD8   SR42, SR43, SR44, SR45, SR46, SR47, SR4A, SR4B, SR4C ;

    /*   extended CRTC registers */
    CARD8   CR13, CR30, CR31, CR32, CR33, CR34, CR35, CR36;
    CARD8   CR37, CR38, CR39, CR3A, CR40, CR41, CR42, CR43;
    CARD8   CR44, CR45, CR46, CR47, CR48, CR49, CR4A;
    CARD8   CRTCRegs[68];

    CARD8   CR97, CR99;

    char *VGAFB;
} VIARegRec, *VIARegPtr;

typedef struct _twodContext {
    CARD32 mode;
} ViaTwodContext;

typedef struct{
    unsigned curPos;
    CARD32 buffer[VIA_CBUFFERSIZE];
    int status;
} ViaCBuffer;

typedef struct VIARec {
    int			scrnIndex;
    VIARegRec           SavedReg;

    /* These are physical addresses. */
    unsigned long       FrameBufferBase;
    unsigned long       MmioBase;

    /* These are linear addresses. */
    unsigned char*      MapBase;
    unsigned char*      BltBase;
    unsigned char*      FBBase;

    CARD8               MemType;
    CARD32              Bandwidth; /* available memory bandwidth */

    /* Handle direct CPU->FB access */
    Bool                FBDirectOption; /* should we force something? */
    Bool                FBDirect; /* set or not? */

    /* group all rudimentary memory management together. */
    unsigned int        FBFreeStart;
    unsigned int        FBFreeSize;

    /* the various bits of framebuffer that got assigned. */
    unsigned int        FBCursorStart;
    unsigned int        FBCursorSize;
    unsigned int        FBScanoutStart;
    unsigned int        FBScanoutSize;
    unsigned int        FBOffscreenStart;
    unsigned int        FBOffscreenSize;
    unsigned int        FBVQStart;
    unsigned int        FBVQSize;

    /* Here are all the Options */
    Bool                VQEnable;
    Bool                hwcursor;
    enum VIAAccelMethod AccelMethod;
    int                 rotate;

    CloseScreenProcPtr  CloseScreen;
    pciVideoPtr         PciInfo;
#ifndef XSERVER_LIBPCIACCESS
    PCITAG              PciTag;
#endif
    int                 Chipset;
    int                 ChipId;
    int                 Host;
    CARD8               HostRev;

    /* Support for shadowFB and rotation */
    unsigned char*      ShadowPtr;
    int                 ShadowPitch;
    void                (*PointerMoved)(int index, int x, int y);

    /* Support for XAA acceleration */
    struct _XAAInfoRec  *AccelInfoRec;
    CARD32              RegPitch; /* used _very_ often */
    xRectangle          Rect;
    CARD32              SavedCmd;
    CARD32              SavedFgColor;
    CARD32              SavedBgColor;
    CARD32              SavedPattern0;
    CARD32              SavedPattern1;
    CARD32              SavedPatternAddr;
    int                 justSetup;
    ViaTwodContext      td;
    ViaCBuffer          cBuf;

    struct ViaScratch   *Scratch;
    struct ViaCardId    *Id;

    Bool                Simultaneous;
    struct ViaCrtc      *Crtc[2];
    struct ViaOutput    *Outputs;

    /* Track CRTC Scaling for overlays */
    CARD16              VisibleWidth;
    CARD16              VisibleHeight;
    CARD16              ScaledWidth;
    CARD16              ScaledHeight;

    DisplayModePtr      XModes; /* X's original modes */

    /* Move this to Crtc->Private */
    Bool                Crtc2HScaleInterpolate;
    Bool                Crtc2VScaleInterpolate;

#define I2C_BUS_COUNT 5
   struct _I2CBusRec    *I2CBus[I2C_BUS_COUNT];

#ifdef HAVE_DRI
    Bool                UseDRI;
    Bool		directRenderingEnabled;
    int 		drmFD;
    struct ViaDri       *Dri;
#endif
    Bool		DRIIrqEnable;

    Bool                CommandBufferActive;
    Bool                UseCommandBuffer;
    Bool                UseCommandBufferFor2d;
    Bool                UseDMACopy;

    CARD8               ActiveDevice;	/* Option */

    struct ViaCursor    *Cursor;

    /* Video */
#ifdef _VIA_VIDEO_H_
    Bool                UseXv;
    Bool                SwovCrtc2;
    struct ViaSwov *    Swov;
#endif
} *VIAPtr;

#define VIAPTR(p) ((VIAPtr)((p)->driverPrivate))

/* Prototypes. */
#ifdef HAVE_DRI
void VIAInitialize3DEngine(ScrnInfoPtr pScrn);
#endif

/* In via_cursor.c. */
Bool VIACursorFBGrab(ScrnInfoPtr pScrn);
struct ViaCursor *VIACursorInit(ScrnInfoPtr pScrn, ScreenPtr pScreen);
void VIACursorDestroy(VIAPtr pVia);
void VIACursorStore(VIAPtr pVia);
void VIACursorRestore(VIAPtr pVia);
void VIACursorHide(VIAPtr pVia);

/* In via_accel.c. */
Bool VIAVQFBGrab(ScrnInfoPtr pScrn);
void ViaVQEnable(ScrnInfoPtr pScrn);
void ViaVQDisable(ScrnInfoPtr pScrn);

void VIAAccelFBGrab(ScrnInfoPtr pScrn);
Bool VIAInitAccel(ScrnInfoPtr pScrn, ScreenPtr pScreen);
void VIAInitialize2DEngine(ScrnInfoPtr pScrn);
void VIAAccelSync(ScrnInfoPtr pScrn);

/* in via_host.c */
void ViaHostIdentify(ScrnInfoPtr pScrn);
Bool ViaFBInit(ScrnInfoPtr pScrn);
void ViaFBBaseGet(ScrnInfoPtr pScrn);

/* In via_shadow.c */
void ViaShadowFBInit(ScrnInfoPtr pScrn, ScreenPtr pScreen);

/* In via_video.c */
#ifdef _VIA_VIDEO_H_
void ViaVideoInit(ScrnInfoPtr pScrn, ScreenPtr pScreen);
void ViaVideoDestroy(ScrnInfoPtr pScrn);
#endif

/* via_i2c.c */
void ViaI2CInit(ScrnInfoPtr pScrn);

/* via_dri.c */
#ifdef HAVE_DRI
Bool VIADRIScreenInit(ScrnInfoPtr pScrn, ScreenPtr pScreen);
void VIADRICloseScreen(ScrnInfoPtr pScrn, ScreenPtr pScreen);
Bool VIADRIFinishScreenInit(ScrnInfoPtr pScrn, ScreenPtr pScreen);

Bool VIADRMCommandBufferStart(VIAPtr pVia);
void VIADRMCommandBufferStop(VIAPtr pVia);
#endif /* HAVE_DRI */

/* Extra debug verbosity */

/* __func__ is really nice, but not universal */
#if !defined(__GNUC__) && !defined(C99)
#define __func__ "unknown"
#endif

#define LOG_DEBUG 7
#define LOG_DEBUG_XV LOG_DEBUG + 1

void ViaDebug(int scrnIndex, const char *format, ...);
#define VIAFUNCI(scrnIndex) (ViaDebug((scrnIndex), "FUNCTION: %s\n", __func__))
#define VIAFUNC(p) VIAFUNCI((p)->scrnIndex)

#endif /* _VIA_DRIVER_H_ */
