/*
 * Copyright 2006-2009 Luc Verhaegen <libv@skynet.be>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * Unichrome MPEG acceleration, driver side support.
 *
 */
#ifndef HAVE_VIA_MPEG_H
#define HAVE_VIA_MPEG_H 1

/* Enable this to have extra debugging in the Xorg log. */
#if 0
#define MPEGDebug(x) (x)
#else
#define MPEGDebug(x)
#endif

#define FOURCC_MPEG (('G' << 24) | ('E' << 16) | ('P' << 8) | 'M')

/* xine seems to need at least 8, engine can only handle 4,
 * so we keep the others in limbo in the FB. */
#define VIAMPEG_BUFFERS_MAX 16

struct ViaMpeg {
    int scrnIndex;

    XID ContextID; /* The XvMC way. */
    CARD16 Width;
    CARD16 Height;
    CARD16 MBWidth;
    CARD16 MBHeight;

    XID CurrentID;
    XID FlippingID;
    XID FRefID;
    XID BRefID;

    struct {
	XID ID;
        struct ViaMem *Mem;
    } Buffers[VIAMPEG_BUFFERS_MAX];

    /* Provide direct register access */
    struct ViaMpegRegs *Regs;
};

/* from via_mpeg.c, to allow via_video.c to sync this engine */
void ViaMpegEngineIdle(struct ViaMpeg *Mpeg);

/* from via_video.c, to allow via_mpeg.c to sync this engine */
void ViaHQVIdle(struct ViaSwov *Swov);

void ViaMpegInit(ScrnInfoPtr pScrn);
void ViaMpegStop(ScrnInfoPtr pScrn);
void ViaMpegDestroy(ScrnInfoPtr pScrn);
Bool ViaMpegBufferIDVerify(struct ViaMpeg *Mpeg, XID BufferID,
			   int Width, int Height);
unsigned long ViaMpegBufferFlip(struct ViaMpeg *Mpeg, XID BufferID);
void ViaMpegBufferFlipRelease(struct ViaMpeg *Mpeg);

/*
 * Mpeg support for XvMC(E).
 */
Bool ViaMpegEngineInit(struct ViaMpeg *Mpeg, CARD16 Width, CARD16 Height);
Bool ViaMpegBufferGrab(struct ViaMpeg *Mpeg, XID BufferID);
void ViaMpegBufferRelease(struct ViaMpeg *Mpeg, XID BufferID);

#define MPEG_PICTURE_FIELD_TOP 0
#define MPEG_PICTURE_FIELD_BOTTOM 1
#define MPEG_PICTURE_FRAME 2
#define MPEG_FRAMETYPE_I 0
#define MPEG_FRAMETYPE_P 1
#define MPEG_FRAMETYPE_B 2
int ViaMpegSliceInit(struct ViaMpeg *Mpeg, XID BufferID, XID FRefID, XID BRefID,
		     CARD8 F_MV_Range, CARD8 B_MV_Range, CARD8 PictureStructure,
		     CARD8 FrameType, Bool ScanningAlternate, Bool PredictionFrame,
		     Bool TopFieldFirst, Bool IntraHasMV, Bool UseQScaleTableType1,
		     CARD8 IntraDCPrecision, Bool UseIntraDCTable1);
int ViaMpegSlicePut(struct ViaMpeg *Mpeg, XID BufferID, CARD8 SliceCode,
		    CARD32 SliceLength, CARD8 *Slice);
void ViaMpegQMatrix(struct ViaMpeg *Mpeg, CARD32 Type, CARD8 Matrix[64]);

#endif /* HAVE_VIA_MPEG_H */
