/*
 * Copyright 2006 Luc Verhaegen <libv@skynet.be>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/*
 * Unichrome MPEG register description.
 *
 * This file is needed by both the client library and the driver.
 *
 * TODO: version checking for this information.
 */
#ifndef HAVE_VIA_MPEGREGS_H
#define HAVE_VIA_MPEGREGS_H 1

/*
 * MPEG engine addresses, for VT3122, VT7205, VT3108.
 * For use in Command Stream.
 */
#define MPEG_PICTURE         0x00
#define MPEG_MACROBLOCK      0x04
#define MPEG_NULL            0x08
#define MPEG_CONTROL         0x0C
/* Motion Vectors: from 0x10 till 0x20; 0 <= i <= 3 */
#define MPEG_MV(i)           (0x10 + 0x4*(i))
/* Buffer addresses: from 0x20 till 0x50;  0 <= i <= 3 */
#define MPEG_BUFFER_Y(i)     (0x20 + 0xC*(i))
#define MPEG_BUFFER_CB(i)    (0x24 + 0xC*(i))
#define MPEG_BUFFER_CR(i)    (0x28 + 0xC*(i))
#define MPEG_LINEOFFSET      0x50
#define MPEG_STATUS          0x54
#define MPEG_IDCTBLOCK       0x58
#define MPEG_QMATRIXSELECT   0x5C
#define MPEG_QMATRIXCONTENT  0x60
#define MPEG_SLICESTREAMOUT  0x64
/* gap */
/* IDCT coefficients: from 0x80 till 0x90; 0 <= i <= 3 */
#define MPEG_IDCTDATA(i)     (0x80 + 0x4*(i))
#define MPEG_SLICECONTROL1   0x90
#define MPEG_SLICECONTROL2   0x94
#define MPEG_SLICECONTROL3   0x98
#define MPEG_SLICECONTROL4   0x9C
/* Slice stream input: from 0xA0 till 0xB4: 0 <= i <= 4 (???) */
#define MPEG_SLICESTREAM(i)  (0xA0 + 0x4*(i))

/*
 * Mpeg Register structure, for direct access.
 */
struct ViaMpegRegs {
    volatile CARD32 Picture;    /* 0x00 */
    volatile CARD32 Macroblock; /* 0x04 */
    volatile CARD32 Null;       /* 0x08 */
    volatile CARD32 Control;    /* 0x0C */
    volatile CARD32 MV[4];      /* 0x10 - 0x20 */
    struct {
        volatile CARD32 Y;
        volatile CARD32 Cb;
        volatile CARD32 Cr;
    } Buffer[4];                /* 0x20 - 0x50 */
    volatile CARD32 LineOffset;     /* 0x50 */
    volatile CARD32 Status;         /* 0x54 */
    volatile CARD32 IDCTBlock;      /* 0x58 */
    volatile CARD32 QMatrixSelect;  /* 0x5C */
    volatile CARD32 QMatrixContent; /* 0x60 */
    volatile CARD32 SliceStreamOut; /* 0x64 */
    volatile CARD32 Reserved[6];    /* 0x68 - 0x80 */
    volatile CARD32 IDCTData[4];    /* 0x80 - 0x90 */
    volatile CARD32 SliceControl1;  /* 0x90 */
    volatile CARD32 SliceControl2;  /* 0x94 */
    volatile CARD32 SliceControl3;  /* 0x98 */
    volatile CARD32 SliceControl4;  /* 0x9C */
    volatile CARD32 SliceStream[5]; /* 0xA0 */
};

/*
 *
 * SubPicture Support.
 *
 * Base Address: 0x3C0
 */

/* SUBPICTURE Registers */
#define SUBPICTURE_CONTROL         0x00
#define SUBPICTURE_ADDRESS         0x04
#define SUBPICTURE_PALETTE         0x08
#define SUBPICTURE_PALETTE_READ    0x0C

/* SUBPICTURE_CONTROL */
#define SUBPICTURE_HQV_ENABLE      0x00010000
#define SUBPICTURE_IA44            0x00020000
#define SUBPICTURE_STRIDE          0x00001fff

struct ViaSubPictureRegs {
    volatile CARD32 Control;     /* 0x00 */
    volatile CARD32 Address;     /* 0x04 */
    volatile CARD32 Palette;     /* 0x08 */
    volatile CARD32 PaletteRead; /* 0x0C */
};

#endif /* HAVE_VIA_MPEGREGS_H */
